#include <gui/scene_manager.h>
#include <applications.h>
#include <furi_hal.h>
#include <toolbox/saved_struct.h>
#include <stdbool.h>
#include <loader/loader.h>

#include "../desktop_i.h"
#include "../desktop_settings/desktop_settings.h"
#include "../views/desktop_view_lock_menu.h"
#include "desktop_scene_i.h"
#include "desktop_scene.h"

#define TAG "DesktopSceneLock"

void desktop_scene_lock_menu_callback(DesktopEvent event, void* context) {
    Desktop* desktop = (Desktop*)context;
    view_dispatcher_send_custom_event(desktop->view_dispatcher, event);
}

void desktop_scene_lock_menu_on_enter(void* context) {
    Desktop* desktop = (Desktop*)context;

    LOAD_DESKTOP_SETTINGS(&desktop->settings);
    scene_manager_set_scene_state(desktop->scene_manager, DesktopSceneLockMenu, 0);
    desktop_lock_menu_set_callback(desktop->lock_menu, desktop_scene_lock_menu_callback, desktop);
    desktop_lock_menu_pin_set(desktop->lock_menu, desktop->settings.pin_code.length > 0);
    desktop_lock_menu_set_idx(desktop->lock_menu, 0);

    view_dispatcher_switch_to_view(desktop->view_dispatcher, DesktopViewIdLockMenu);
}

bool desktop_scene_lock_menu_on_event(void* context, SceneManagerEvent event) {
    Desktop* desktop = (Desktop*)context;
    bool consumed = false;

    if(event.type == SceneManagerEventTypeTick) {
        bool check_pin_changed =
            scene_manager_get_scene_state(desktop->scene_manager, DesktopSceneLockMenu);
        if(check_pin_changed) {
            LOAD_DESKTOP_SETTINGS(&desktop->settings);
            if(desktop->settings.pin_code.length > 0) {
                desktop_lock_menu_pin_set(desktop->lock_menu, 1);
                scene_manager_set_scene_state(desktop->scene_manager, DesktopSceneLockMenu, 0);
            }
        }
    } else if(event.type == SceneManagerEventTypeCustom) {
        switch(event.event) {
        case DesktopLockMenuEventLock:
            scene_manager_set_scene_state(desktop->scene_manager, DesktopSceneLockMenu, 0);
            scene_manager_set_scene_state(
                desktop->scene_manager, DesktopSceneLocked, SCENE_LOCKED_FIRST_ENTER);
            scene_manager_next_scene(desktop->scene_manager, DesktopSceneLocked);
            consumed = true;
            break;
        case DesktopLockMenuEventPinLock:
            if(desktop->settings.pin_code.length > 0) {
                furi_hal_rtc_set_flag(FuriHalRtcFlagLock);
                scene_manager_set_scene_state(
                    desktop->scene_manager, DesktopSceneLocked, SCENE_LOCKED_FIRST_ENTER);
                scene_manager_next_scene(desktop->scene_manager, DesktopSceneLocked);
            } else {
                Loader* loader = furi_record_open("loader");
                LoaderStatus status =
                    loader_start(loader, "Desktop", DESKTOP_SETTINGS_RUN_PIN_SETUP_ARG);
                furi_record_close("loader");
                if(status == LoaderStatusOk) {
                    scene_manager_set_scene_state(desktop->scene_manager, DesktopSceneLockMenu, 1);
                } else {
                    FURI_LOG_E(TAG, "Unable to start desktop settings");
                }
            }

            consumed = true;
            break;
        case DesktopLockMenuEventExit:
            scene_manager_set_scene_state(desktop->scene_manager, DesktopSceneLockMenu, 0);
            scene_manager_search_and_switch_to_previous_scene(
                desktop->scene_manager, DesktopSceneMain);
            consumed = true;
            break;
        default:
            break;
        }
    }
    return consumed;
}

void desktop_scene_lock_menu_on_exit(void* context) {
}
